<div class="frame {frame_class}">
  <div class="frame-name">
    <span class="expander">▼</span>
    <span class="breakable">{file}</span> in <span>{symbol_name}</span> at line
    <span>{line}</span>
  </div>
  <div class="code-snippet-wrapper">
    <table role="presentation" class="code-snippet">
      {code}
    </table>
  </div>
</div>
